@@ -95,7 +95,7 @@ class StringArg(ArgType):
 	else:
             info.add_parselist('s', ['&' + pname], [pname])
     def write_return(self, ptype, info):
-	if ptype in ('const-gchar*', 'const-char*', 'static_string'):
+        if ptype[:6] == 'const-' or ptype in ('static_string',):
 	    info.varlist.add('const gchar', '*ret')
             info.codeafter.append('    if (ret)\n' +
                                   '        return PyString_FromString(ret);\n'+
@@ -128,24 +128,6 @@ class UCharArg(ArgType):
             info.add_parselist('s#', ['&' + pname, '&' + pname + '_len'],
                                [pname])
 
-    def write_return(self, ptype, info):
-        if ptype.startswith('const-'):
-	    info.varlist.add('const guchar', '*ret')
-            info.codeafter.append('    if (ret)\n' +
-                                  '        return PyUnicode_DeviceUTF8(ret, strlen(ret), "strict");\n'+
-                                  '    Py_INCREF(Py_None);\n' +
-                                  '    return Py_None;')
-	else:
-	    # have to free result ...
-	    info.varlist.add('guchar', '*ret')
-            info.codeafter.append('    if (ret) {\n' +
-                                  '        PyObject *py_ret = PyUnicode_DecodeUTF8(ret, strlen(ret), "strict");\n' +
-                                  '        g_free(ret);\n' +
-                                  '        return py_ret;\n' +
-                                  '    }\n' +
-                                  '    Py_INCREF(Py_None);\n' +
-                                  '    return Py_None;')
-
 class CharArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
 	if pdflt:
@@ -416,6 +398,48 @@ class CustomBoxedArg(ArgType):
                               '    Py_INCREF(Py_None);\n' +
                               '    return Py_None;')
 
+class PointerArg(ArgType):
+    # haven't done support for default args.  Is it needed?
+    check = ('    if (pyg_pointer_check(py_%(name)s, %(typecode)s))\n'
+             '        %(name)s = pyg_pointer_get(py_%(name)s, %(typename)s);\n'
+             '    else {\n'
+             '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a %(typename)s");\n'
+             '        return NULL;\n'
+             '    }\n')
+    null = ('    if (pyg_pointer_check(py_%(name)s, %(typecode)s))\n'
+            '        %(name)s = pyg_pointer_get(py_%(name)s, %(typename)s);\n'
+            '    else if (py_%(name)s != Py_None) {\n'
+            '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a %(typename)s or None");\n'
+            '        return NULL;\n'
+            '    }\n')
+    def __init__(self, ptype, typecode):
+	self.typename = ptype
+	self.typecode = typecode
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+	if pnull:
+            info.varlist.add(self.typename, '*' + pname + ' = NULL')
+	    info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
+	    info.codebefore.append(self.null % {'name':  pname,
+                                                'typename': self.typename,
+                                                'typecode': self.typecode})
+	else:
+            info.varlist.add(self.typename, '*' + pname + ' = NULL')
+	    info.varlist.add('PyObject', '*py_' + pname)
+	    info.codebefore.append(self.check % {'name':  pname,
+                                                 'typename': self.typename,
+                                                 'typecode': self.typecode})
+        info.arglist.append(pname)
+        info.add_parselist('O', ['&py_' + pname], [pname])
+    def write_return(self, ptype, info):
+        if ptype[-1] == '*':
+            info.varlist.add(self.typename, '*ret')
+            info.codeafter.append('    /* pyg_pointer_new handles NULL checking */\n' +
+                                  '    return pyg_pointer_new(' + self.typecode + ', ret);')
+        else:
+            info.varlist.add(self.typename, 'ret')
+            info.codeafter.append('    /* pyg_pointer_new handles NULL checking */\n' +
+                                  '    return pyg_pointer_new(' + self.typecode + ', &ret);')
+
 class AtomArg(IntArg):
     atom = ('    %(name)s = pygdk_atom_from_pyobject(py_%(name)s);\n'
             '    if (PyErr_Occurred())\n'
@@ -454,7 +478,7 @@ class GErrorArg(ArgType):
 
 class GtkTreePathArg(ArgType):
     # haven't done support for default args.  Is it needed?
-    normal = '    %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);'
+    normal = '    %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
     null = ('    if (PyTuple_Check(py_%(name)s))\n'
             '        %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
             '    else if (py_%(name)s != Py_None) {\n'
@@ -462,7 +486,7 @@ class GtkTreePathArg(ArgType):
             '        return NULL;\n'
             '    }\n')
     freepath = ('    if (%(name)s)\n'
-                '        gtk_tree_path_free(%(name)s);')
+                '        gtk_tree_path_free(%(name)s);\n')
     def __init__(self):
         pass
     def write_param(self, ptype, pname, pdflt, pnull, info):
@@ -488,7 +512,34 @@ class GtkTreePathArg(ArgType):
                               '    }\n' +
                               '    Py_INCREF(Py_None);\n' +
                               '    return Py_None;')
+													       
+						       
+class GdkRectanglePtrArg(ArgType):
+    null = ('    if (PyTuple_Check(py_%(name)s)) {\n'
+            '        %(name)s = g_new(GdkRectangle, 1);\n'
+	    '        if (!PyArg_ParseTuple(py_%(name)s, "iiii", &%(name)s->x, &%(name)s->y, &%(name)s->width, &%(name)s->height)) {\n'
+	    '            return NULL;\n'
+	    '        }\n'
+	    '    } else {\n'
+	    '        PyErr_SetString(PyExc_TypeError, "%(name)s must be a tuple");\n'
+	    '    }\n')
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+	if pnull:
+	    info.varlist.add('GdkRectangle', '*' + pname + ' = NULL')
+	    info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
+            info.add_parselist('O', ['&py_' + pname], [pname])
+	else:
+	    info.varlist.add('GdkRectangle', '*' + pname + ' = NULL')
+	    info.varlist.add('PyObject', '*py_' + pname)
+            info.add_parselist('O!', ['&PyTuple_Type', '&py_' + pname],[pname])
+	info.arglist.append(pname)
+	info.codebefore.append(self.null % {'name':  pname})
 
+class GdkRectangleArg(ArgType):
+    def write_return(self, ptype, info):
+	info.varlist.add('GdkRectangle', 'ret')
+	info.codeafter.append('    return pyg_boxed_new(GDK_RECTANGLE, ret, TRUE, TRUE);')
+	
 class ArgMatcher:
     def __init__(self):
 	self.argtypes = {}
@@ -520,6 +571,11 @@ class ArgMatcher:
         arg = CustomBoxedArg(ptype, pytype, getter, new)
 	self.register(ptype+'*', arg)
         self.register('const-'+ptype+'*', arg)
+    def register_pointer(self, ptype, typecode):
+        arg = PointerArg(ptype, typecode)
+        self.register(ptype, arg)
+	self.register(ptype+'*', arg)
+        self.register('const-'+ptype+'*', arg)
 
     def get(self, ptype):
 	return self.argtypes[ptype]
@@ -591,6 +647,8 @@ matcher.register('GtkType', GTypeArg())
 
 matcher.register('GError**', GErrorArg())
 matcher.register('GtkTreePath*', GtkTreePathArg())
+matcher.register('GdkRectangle*', GdkRectanglePtrArg())
+matcher.register('GdkRectangle', GdkRectangleArg())
 
 matcher.register_object('GObject', None, 'G_TYPE_OBJECT')
 
