@@ -1320,6 +1320,10 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pyg_register_boxed,
   pyg_boxed_new,
 
+  &PyGPointer_Type,
+  pyg_register_pointer,
+  pyg_pointer_new,
+
   pyg_enum_add_constants,
   pyg_flags_add_constants,
   
@@ -1382,6 +1386,14 @@ initgobject(void)
 			 o=pyg_type_wrapper_new(G_TYPE_BOXED));
     Py_DECREF(o);
 
+    PyGPointer_Type.ob_type = &PyType_Type;
+    if (PyType_Ready(&PyGPointer_Type))
+	return;
+    PyDict_SetItemString(d, "GPointer", (PyObject *)&PyGPointer_Type);
+    PyDict_SetItemString(PyGPointer_Type.tp_dict, "__gtype__",
+			 o=pyg_type_wrapper_new(G_TYPE_POINTER));
+    Py_DECREF(o);
+
     tuple = Py_BuildValue ("(iii)", glib_major_version, glib_minor_version,
 			   glib_micro_version);
     PyDict_SetItemString(d, "glib_version", tuple);    
@@ -1389,8 +1401,9 @@ initgobject(void)
 
     /* for addon libraries ... */
     PyDict_SetItemString(d, "_PyGObject_API",
-			 PyCObject_FromVoidPtr(&pygobject_api_functions,NULL));
-
+			 o=PyCObject_FromVoidPtr(&pygobject_api_functions,NULL));
+    Py_DECREF(o);
+	
     /* some constants */
     PyModule_AddIntConstant(m, "SIGNAL_RUN_FIRST", G_SIGNAL_RUN_FIRST);
     PyModule_AddIntConstant(m, "SIGNAL_RUN_LAST", G_SIGNAL_RUN_LAST);
