@@ -26,7 +26,7 @@ static struct _PyGtk_FunctionStruct functions = {
 DL_EXPORT(void)
 init_gtk(void)
 {
-    PyObject *m, *d, *tuple;
+    PyObject *m, *d, *tuple, *o;
     PyObject *av;
     int argc, i;
     char **argv;
@@ -41,10 +41,16 @@ init_gtk(void)
 
     /* initialise GTK ... */
     av = PySys_GetObject("argv");
-    argc = PyList_Size(av);
-    argv = g_new(char *, argc);
-    for (i = 0; i < argc; i++)
-	argv[i] = g_strdup(PyString_AsString(PyList_GetItem(av, i)));
+    if (av != NULL) {
+	argc = PyList_Size(av);
+	argv = g_new(char *, argc);
+	for (i = 0; i < argc; i++)
+	    argv[i] = g_strdup(PyString_AsString(PyList_GetItem(av, i)));
+    } else {
+	    argc = 0;
+	    argv = NULL;
+    }
+	
     if (!gtk_init_check(&argc, &argv)) {
 	if (argv != NULL) {
 	    for (i = 0; i < argc; i++)
@@ -54,8 +60,8 @@ init_gtk(void)
 	PyErr_SetString(PyExc_RuntimeError, "could not open display");
 	return;
     }
-    PySys_SetArgv(argc, argv);
     if (argv != NULL) {
+	PySys_SetArgv(argc, argv);
 	for (i = 0; i < argc; i++)
 	    g_free(argv[i]);
 	g_free(argv);
@@ -83,8 +89,9 @@ init_gtk(void)
     
     /* for addon libraries ... */
     PyDict_SetItemString(d, "_PyGtk_API",
-			 PyCObject_FromVoidPtr(&functions, NULL));
-
+			 o=PyCObject_FromVoidPtr(&functions, NULL));
+    Py_DECREF(o);
+	
     stock_ids = gtk_stock_list_ids();
     strcpy(buf, "STOCK_");
     for (cur = stock_ids; cur; cur = stock_ids) {
@@ -93,8 +100,12 @@ init_gtk(void)
 	int i;
 	
 	ctmp = cur->data;
-	if(strncmp(ctmp, "gtk-", 4))
-	    continue;
+	if(strncmp(ctmp, "gtk-", 4)) {
+		g_free(cur->data);
+		stock_ids = cur->next;
+		g_slist_free_1(cur);
+		continue;
+	}
 	ctmp += 4;
 	
 	strcpy(buf + sizeof("STOCK"), ctmp);
